<?php

## -- Carousel.php  

##    version 2022-05-06
##
##  Drop-In automatic slide-show using all images in a specified directory.
##  Built from the example code at
##    https://www.w3schools.com/howto/howto_js_slideshow.asp
##
##  Adapted and modified to be a PMWiki recipe, for simple Markup use:
##
##  e.g.  (:Carousel /Gallery/CarouSel3:)
##
##  Copyright 2022 Kirk Siqveland DBA: CyberTamer Services
##    CodeMonkey@CyberTamer.com
##
##  This version added 
##  - autoplay for the slideshow and
##  - automatically finds the correct wiki path to use.
##    (still assumes the slideshow is in the same path as the wiki.
##     e.g. www.myexampledomain.org/pmwiki/media/slideshow )
##    

/*  This recipe is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with your copy of th PMWiki code; if not, write to the 
    Free Software Foundation, Inc., 
    59 Temple Place, Suite 330, 
    Boston, MA  02111-1307  USA
*/


if (!defined('PmWiki')) exit();

SDV($RecipeInfo['Carousel']['Version'], '2022-05-06');
$FmtPV['$CarouselVersion'] = "'{$RecipeInfo["Carousel"]["Version"]}'";


## $WikiRoot is the full server path to the directory where pmwiki.php is installed
    if(empty($WikiRoot)){
          $shortRoot  = str_replace("pmwiki.php", "", $_SERVER['PHP_SELF'] ) ;
          $ScriptURI  = $_SERVER['SCRIPT_URI'] ;
          $ServerName = $_SERVER['SERVER_NAME'] ;
          $Protocol   = substr($ScriptURI, 0, strpos($ScriptURI, $ServerName));

        $WikiRoot   = $Protocol.$ServerName.$shortRoot ;
    }    

## For Debug Purposes you can display what we are getting for $WikiRoot
##     Just use the markup   (:showWikiRoot:)
  Markup(
    'showWikiRoot', 
    'style',
    '/\\(:showWikiRoot:\\)/i',
    "$WikiRoot"
  ) ;    


## Carousel Markup Insert a Carousel with one line of Markup indicating the
## Directory where the images to use can be found 
## This uses a Relative-Path from the Wiki Root so if the folder is
##     www.mydomain.org/pmwiki/media/MySlideShow 
## then we use this Markup:
##   (:Carousel /media/MySlideShow/:)

## for autoplay slide show you have two more options:
## for a default slidechange - currently $slideRate = 3500 milliseconds
##    (:Carousel /media/MySlideShow/ autorun:)
## or
##    define the delay in milliseconds for example 4500:
##    (:Carousel /media/MySlideShow/ autorun 4500:)

  Markup(
    'Carousel', 
    'style',
    '/\\(:Carousel\\s*(.*?):\\)/i',
    "BuildSlideCarousel"
  ) ;


  function BuildSlideCarousel($m){
    global $WikiRoot;

    $params     = trim($m[1]) ;
    $imgTally   = 0 ;
    $autorun    = "false" ;
    $slideRate  = 3500 ;  // Milliseconds so this is 3.5 seconds
    
    ## If we have a space in the parameters then we have more than one...
      if(strpos($params, " ")){
          $paramArr   = explode(" ",$params) ;
          ## the first parameter should be the relative path
          $relPath    = $paramArr[0] ;

          if(array_search("autorun",$paramArr)!==false){
            $autorun = "true" ;
              if( count($paramArr)>2){
                $slideRate = $paramArr[2] ;
              }                  
          }
      }else{  $relPath = $params ;  }

      ## This assumes you are using a relative pathname to a directory 
      ## somewhere in the same domain as your PMWiki installation.
      ## e.g I like to use something like
      ##  myexampledomain.org/pmwiki/media/slideshow

      ## for this part we need the server path name not the URL
      ## Assuming this script is run from /cookbook we get the parent of /cookbook
      ## Which should be the root path for the wiki, whatever it may be named.
      $dir  = dirname(__DIR__) ; ## local path e.g. /home/username/mywikidomain.org/pmwiki/
                                 ## this is the path of the pmwiki.php script
      $dir .= $relPath         ; ##                 


      if (is_dir( $dir ) ){
        ## - Scan the directory for all files then extract images and build table.
        ##   Open directory | Read Line-BY-Line into $dirArray | Close directory
          $myDirectory = opendir($dir) ;
              while($entryName = readdir($myDirectory))
              { 
                if(!is_dir( $entryName )){  
                    // Find Image files and add them to the list (ignore directories)
                        // Get file extension
                          $extn=pathinfo($entryName, PATHINFO_EXTENSION) ; 
                    if($extn == "jpg"||$extn == "jpeg"||$extn == "png"||$extn == "gif"){
                      $dirArray[] = $entryName ;  
                    } 
                }
              }
          closedir($myDirectory) ;  
      }else{
          $badDir = true;
      }

      if (!$badDir ) 
      {
        ## Count elements in array
          $indexCount=count($dirArray) ;

        ## Sort directory file list array
          sort($dirArray) ;  

        ## Now build the Carousel HTML:
          $RawCarouSel  = "<!-- Carousel container -->";
          $RawCarouSel .= "<div class='carousel-container'>"; 

          for($index=0 ; $index < $indexCount ; $index++) 
          {
            ## Get File / directory Names
            $name = $dirArray[$index] ;  

                ## add html for each image in the directory, hiding all but the first 
                      $tally++;
                      $RawCarouSel .= "  <!-- Full-width images with number and caption text -->\n";
                        if($tally==1){
                            $RawCarouSel .= "  <div class='mySlides'";
                        }else{
                            $RawCarouSel .= "  <div class='mySlides hidden'";
                        }
                      $RawCarouSel .= " id='Slide".$tally."'>\n";

##OPTION>>            ## Display slide number / out-of-total  e.g.:  3 / 10
                        $RawCarouSel .= "    <div class='numbertext'>".$tally." / ".$indexCount."</div>\n";
                  
                      ## Build the Image source code:
                        $RawCarouSel .= "    <img src='" ;
                             
                        ## Insert the image path and name (Here we need the URL not the Server Path)
                              $fullpath  = $WikiRoot.$relPath ;      
                          $RawCarouSel .= $fullpath.$name ."' style='width:100%'>\n" ;

                      ## Now we get the display name for the image
                      ## This allows for naming of slides using numeric order:
                      ##    eg 5_-_Basel_Switzerland
                          $peg = strpos($name, "_-_");
                          if($peg){
                            $pegTitle = substr($name, $peg+3);
                            $imgTitle  = str_replace(array(".jpg", ".jpeg", ".png", ".gif"),"",$pegTitle) ;
                          }else{
                            $imgTitle  = str_replace(array(".jpg", ".jpeg", ".png", ".gif"),"",$name) ;                              
                          }

                        $imgTitle  = str_replace("_"," ",$imgTitle) ;
                            
##OPTION>>            ## Display the Image name as a Title on the Slide
                        $RawCarouSel .= "    <div class='text'>".$imgTitle."</div>\n";

                      ## Close the '.mySlides' div
                        $RawCarouSel .= "  </div>\n";         
          }


          $RawCarouSel .= "  <!-- Next and previous buttons -->";
          $RawCarouSel .= " <button id='rwd' class='prev hidden'>&#10094;</button> ";
          $RawCarouSel .= " <button id='fwd' class='next hidden'>&#10095;</button> ";

          $RawCarouSel .= "</div>";
          $RawCarouSel .= "<br>";

          $RawCarouSel .= '<script>
                              var slideIndex = 0 ;
                              var slideRate  = '. $slideRate . ' ;
                              var autoRun    = '. $autorun   . ' ;
                              var TimeOut    ;

                              window.onload = function() {
                                  document.getElementById("rwd").addEventListener("click", prevSlide);
                                  document.getElementById("rwd").classList.remove("hidden");
                                  document.getElementById("fwd").addEventListener("click", nextSlide);
                                  document.getElementById("fwd").classList.remove("hidden");    
                              };

                              function showSlides(n) {
                                var i;
                                var slides = document.getElementsByClassName("mySlides");
                                if (n >= slides.length) {slideIndex = 0}
                                if (n <= 0) {slideIndex = slides.length-1}
                                for (i = 0; i < slides.length; i++) {
                                    slides[i].classList.add("hidden");
                                }
                                slides[slideIndex].classList.remove("hidden");
                                //console.log( "Advance " + n);
                              }   

                              function prevSlide() {
                                //alert("prevSlide Clicked ") ;
                                showSlides(slideIndex -= 1)   ;
                              }
                              function nextSlide() {
                                //alert("nextSlide Clicked ") ;
                                showSlides(slideIndex += 1)   ;
                              }

                              function autoAdvance(){
                                showSlides(slideIndex += 1)   ;

                                TimeOut = setTimeout(autoAdvance, slideRate) ;
                              }
                                 
                              if(autoRun){
                                TimeOut = setTimeout(autoAdvance, slideRate) ;
                              }                 
                            </script><br/>';

          return $RawCarouSel; 
      }else{
          return "Unfortunately, we can't find or open this directory:  ".$dir; 
      }
  }

